Editoria11y (editorial accessibility ally) is a user-friendly accessibility checker that addresses three critical needs for content authors:

1. It runs automatically. Modern spellcheck works so well because it is always running; put spellcheck behind a button and few users remember to run it!
1. It focuses exclusively on straightforward issues a content author can easily understand and easily fix. Comprehensive testing should be a key part of site creation, but if a tool is going to run automatically, it will drive an author bonkers if it is constantly alerting on code they do not understand and cannot fix.
1. It runs in context. Views, Layout Builder, Paragraphs and all the other modules Drupal uses to assemble a page means that tools that run inside CKEditor cannot "see" many of the issues on a typical page.

## The authoring experience
* On each page load Editoria11y places a small toggle button at the bottom right of the screen with an issue count. Users can press the button to view details of any alerts or access additional tools ("full check"), including visualizers for the document outline and image alt attributes.
* Depending on configuration settings, the panel may open automatically if new issues are detected.

## Installation and configuration
* On install, Editoria11y adds a new permission ("View Editoria11y Checker"), and grants the permission to any roles which already have common content-editing permissions. This is an inexact science; check to make sure you are happy with the choices.
* By default, Editorially scans the HTML "body" element. Check to see if that makes sense for your site, and override if necessary on the module's configuration page. For many sites, "main" is a better setting.
* By default, Editorially does not run on administrative paths, or if it detects that in-place editing is happening (e.g., Layout Builder). If you have inline editing enabled, check to see if Editoria11y is detecting/ignoring content that is actively being edited. If not, add a selector for elements present during editing to the module's configuration page. And do tell us about the conflict! If it is a common module we will add it to the default ignore list.
* Some content just does not play nice with this type of tool; embedded social media feeds, for example. Add selectors for page elements you want the scanner to ignore.
* If you are installing from the command line, do note the "eleventy" when spelling the module's name!

### Available JS Events

Thrown after Editoria11y activates (after the panel opens and tooltips appear):
```js
document.addEventListener("ed11yPanelOpened", function(event) {
  // jQuery('.example').addClass('editoria11y-active');
});
```

Thrown when a tooltip appears, containing its unique ID:
```js
document.addEventListener("ed11yPop", function(event) {
  let myID = '#' + event.detail.id;
  // jQuery(myID).parents('.example').addClass('has-open-tip');
});
 ```

Thrown for elements listed in the "Theme JS will handle revealing hidden tooltips" configuration option, when the panel's jump link tries to highlight a matching tooltip. After this event is thrown, Editoria11y pauses for half a second, giving your theme a moment to open an accordion or tab panel (etc.) before Editoria11y tries to transfer focus to the hidden tip:
```js
document.addEventListener("ed11yHiddenHandler", function (event) {
  let myID = '#' + event.detail.id;
  // if (jQuery(myID).parents('.example').length > 0) {
  //   jQuery(myID).parents('.example').prev('button').click();
  // }
});
```


## Roadmap

### 1.1 (Spring/Summer '21)
- Further test link and image tooltip placement when elements are floating.
- Evaluate possible additional tests
  - Flag links to dev and QA environments
  - Form field labels
  - Undeclared language detection based on certain words and characters?
  - Contrast tests?
  - Pull visualization alerts into basic check?
- Drupal configuration page
  - Add a variable to control which items are included in the page outline
  - Evaluate adopting content-type config code from Asset Injector
  - Evaluate adding a separate permission to administer Editoria11y
- Look into refactoring panel DOM as well as Drupal code style conformance
  
## Contact
Editoria11y is maintained by [John Jameson](https://www.drupal.org/u/itmaybejj), and is provided to the community thanks to the [Digital Accessibility](https://accessibility.princeton.edu/) initiatives at Princeton University's [Office of Web Development Services](https://wds.princeton.edu/)

## Acknowledgements
Editoria11y's JavaScript began as a fork of the [Sa11y](https://ryersondmp.github.io/sa11y/) accessibility checker, which was created by Digital Media Projects, Computing and Communication Services (CCS) at Ryerson University in Toronto, Canada:
- [Adam Chaboryk](https://github.com/adamchaboryk), IT accessibility specialist
- Benjamin Luong, web accessibility assistant
- Arshad Mohammed, web accessibility assistant
- Kyle Padernilla, web accessibility assistant

Sa11y itself is an adaptation of [Tota11y by Khan Academy](https://github.com/Khan/tota11y), was built with [FontAwesome icons](https://github.com/FortAwesome/Font-Awesome) and is powered with jQuery.
